import StockData from '../StockData';
import CandlestickFinder, { ICandlestickConfig } from './CandlestickFinder';
/**
 * Configuration interface for DownsideTasukiGap pattern.
 * Only requires scale parameter since this pattern uses direct price comparisons.
 */
export interface IDownsideTasukiGapConfig extends ICandlestickConfig {
}
/**
 * Default configuration for DownsideTasukiGap pattern.
 */
export declare const DEFAULT_DOWNSIDE_TASUKI_GAP_CONFIG: IDownsideTasukiGapConfig;
export default class DownsideTasukiGap extends CandlestickFinder {
    constructor(config?: IDownsideTasukiGapConfig);
    logic(data: StockData): boolean;
}
/**
 * Detects DownsideTasukiGap candlestick pattern in the provided stock data.
 *
 * A DownsideTasukiGap is a three-candle continuation pattern that occurs during a downtrend.
 * It consists of:
 * 1. A bearish candle
 * 2. A second bearish candle that gaps down from the first
 * 3. A bullish candle that opens within the second candle's body and closes within the gap
 *
 * This pattern suggests that the downtrend will continue despite the temporary bullish reversal.
 *
 * @param data - Stock data containing OHLC values for at least 3 periods
 * @param config - Configuration object for pattern detection
 * @param config.scale - Scale parameter for approximateEqual function precision (default: 0.001)
 * @returns True if DownsideTasukiGap pattern is detected, false otherwise
 *
 * @example
 * ```typescript
 * // Using default configuration
 * const hasDownsideTasukiGapPattern = downsidetasukigap(stockData);
 *
 * // Using custom configuration
 * const hasDownsideTasukiGapPattern = downsidetasukigap(stockData, {
 *   scale: 0.002
 * });
 *
 * // Backward compatibility with scale parameter
 * const hasDownsideTasukiGapPattern = downsidetasukigap(stockData, { scale: 0.002 });
 * ```
 */
export declare function downsidetasukigap(data: StockData, config?: IDownsideTasukiGapConfig): any;
